﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Shapes;
using TPLSamples.Utility;

namespace TPLSamples.Models
{
	public class AsyncAwaitSample : Sample
	{
		public override string SampleName
		{
			get { return "Async/Await Sample"; }
		}

		public override bool ImageRequired
		{
			get { return false; }
		}

		public async override void Run(System.Drawing.Bitmap bmp = null, Action<string> UpdateLog = null)
		{
			Stopwatch s = new Stopwatch();
			s.Start();

            UpdateLog("Step 1 (Run 1): Starting an await call to an asyncronous method.");
            int result = await AccessTheWebAsync("Run 1", UpdateLog);
            UpdateLog("Step 6 (Run 1): Done await call to an asyncronous method.");


            //this works and will even run asynchronously but won't wait on any result.
            //more then likely we will be long gone from this method before the method below is done
            UpdateLog(Environment.NewLine + "Step 1 (Run 2): Starting an async call without await and no result.");
            AccessTheWebAsync("Run 2", UpdateLog);
            UpdateLog("Step 6 (Run 2): Done with the async call without await and no result");

			s.Stop();
			RunTime = s.Elapsed;
		}

        // Three things to note in the signature: 
        //  - The method has an async modifier.  
        //  - The return type is Task or Task<T>. (See "Return Types" section.)
        //    Here, it is Task<int> because the return statement returns an integer. 
        //  - The method name ends in "Async."
        async Task<int> AccessTheWebAsync(string runDesignation, Action<string> UpdateLog)
        {
            // You need to add a reference to System.Net.Http to declare client.
            HttpClient client = new HttpClient();

            // GetStringAsync returns a Task<string>. That means that when you await the 
            // task you'll get a string (urlContents).
            // This also allows you set a lot of properties on the task rather then just running
            // it if you need to.
            Task<string> getStringTask = client.GetStringAsync("http://msdn.microsoft.com");
            UpdateLog("Step 2 (" + runDesignation + "): Sleeping for ten seconds.");
            await Task.Delay(10000);
            UpdateLog("Step 3 (" + runDesignation + "): Woke up.");
            UpdateLog("Step 4 (" + runDesignation + "): Getting the web page.");
            // The await operator suspends AccessTheWebAsync. 
            //  - AccessTheWebAsync can't continue until getStringTask is complete. 
            //  - Meanwhile, control returns to the caller of AccessTheWebAsync. 
            //  - Control resumes here when getStringTask is complete.  
            //  - The await operator then retrieves the string result from getStringTask. 
            // This could also have been done as 
            // string urlContents = await client.GetStringAsync("http://msdn.microsoft.com");
            string urlContents = await getStringTask;
            UpdateLog("Step 5 (" + runDesignation + "): Got the web page.");
            // The return statement specifies an integer result. 
            // Any methods that are awaiting AccessTheWebAsync retrieve the length value. 
            return urlContents.Length;
        }
	}
}
