﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace TPLSamples.Models
{
    public class LoopBreakSample : Sample
    {
        public override string SampleName
        {
            get { return "Loop Break Sample"; }
        }

        public override bool ImageRequired
        {
            get { return false; }
        }

        protected int MaxValue { get { return 50; } }
        protected int BreakValue { get { return 20; } }

        public override void Run(System.Drawing.Bitmap bmp = null, Action<string> UpdateLog = null)
        {
            Stopwatch s = new Stopwatch();
            s.Start();

            UpdateLog("Running to " + MaxValue);
            var loopResult = Parallel.For(0, MaxValue, (int i, ParallelLoopState loop) =>
            {
                UpdateLog("Starting " + i);
                if (i >= BreakValue)
                {
                    loop.Break();
                    UpdateLog("Breaking " + i);
                    return;
                }

                Thread.Sleep(1000);
            });
            UpdateLog("IsCompleted == " + loopResult.IsCompleted);
            if (!loopResult.LowestBreakIteration.HasValue)
                UpdateLog("LowestBreakIteration has no value");
            else
                UpdateLog("LowestBreakIteration.Value == " + loopResult.LowestBreakIteration.Value);

            s.Stop();
            RunTime = s.Elapsed;
        }
    }
}
