﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TPLSamples.Models
{
    public class MatricesMultiplicationSample : Sample
    {
        // Set up matrices. Use small values to better view 
        // result matrix. Increase the counts to see greater 
		// speedup in the parallel loop vs. the sequential loop.
        protected static readonly int colCount = 180;
        protected static readonly int rowCount = 2000;
        protected static readonly int colCount2 = 270;
        
        //protected statics so these can be used in the other sample
        static readonly Lazy<double[,]> _lazyMatrix1 = new Lazy<double[,]>(() => { return InitializeRandomMatrix(rowCount, colCount); });
        protected static double[,] Matrix1
        {
            get { return _lazyMatrix1.Value; }
        }

        static readonly Lazy<double[,]> _lazyMatrix2 = new Lazy<double[,]>(() => { return InitializeRandomMatrix(colCount, colCount2); });
        protected static double[,] Matrix2
        {
            get { return _lazyMatrix2.Value; }
        }

        static Random ran = new Random();
        static double[,] InitializeRandomMatrix(int rows, int cols)
        {
            double[,] matrix = new double[rows, cols];

            for (int i = 0; i < rows; i++)
            {
                for (int j = 0; j < cols; j++)
                {
                    matrix[i, j] = ran.Next(100);
                }
            }
            return matrix;
        }

        public override string SampleName
        {
            get { return "Matrices Multiplication"; }
        }

        public override bool ImageRequired
        {
            get { return false; }
        }

        public override void Run(System.Drawing.Bitmap bmp = null, Action<string> updateLog = null)
        {
            double[,] matA = Matrix1;
            double[,] matB = Matrix2;
            double[,] result = new double[rowCount, colCount2];

            Stopwatch s = new Stopwatch();
            s.Start();

            int matACols = matA.GetLength(1);
            int matBCols = matB.GetLength(1);
            int matARows = matA.GetLength(0);

            for (int i = 0; i < matARows; i++)
            {
                for (int j = 0; j < matBCols; j++)
                {
                    for (int k = 0; k < matACols; k++)
                    {
                        result[i, j] += matA[i, k] * matB[k, j];
                    }
                }
            }

            s.Stop();
            RunTime = s.Elapsed;
        }
    }
}
