﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Markup;
using Microsoft.Practices.Prism.ViewModel;
using Microsoft.Win32;
using TPLSamples.Models;

[assembly: XmlnsDefinition("http://formatexception.com/tplsamples/viewmodels", "TPLSamples.ViewModels")]
namespace TPLSamples.ViewModels
{
	public class SamplerViewModel : NotificationObject
	{
		public Sampler Sampler
		{
			get;
			set;
		}

		public ObservableCollection<SampleViewModel> Samples { get; private set; }

		private string currentState;
		public string CurrentState
		{
			get
			{
				return this.currentState;
			}

			set
			{
				if (this.currentState != value)
				{
					this.currentState = value;
					this.RaisePropertyChanged(() => this.CurrentState);
				}
			}
		}


		public SamplerViewModel()
		{
			Samples = new ObservableCollection<SampleViewModel>();
			Sampler = new Sampler();
			Sampler.Samples.Add(new LineSample());
			Sampler.Samples.Add(new LineParallelSample());
			Sampler.Samples.Add(new GreyScaleSample());
			Sampler.Samples.Add(new GreyScaleParallelSample());
			Sampler.Samples.Add(new GreyScaleDoubleParallelSample());
            Sampler.Samples.Add(new MatricesMultiplicationSample());
            Sampler.Samples.Add(new MatricesMultiplicationParallelSample());
            Sampler.Samples.Add(new AggregateExceptionNoCatchSample());
            Sampler.Samples.Add(new AggregateExceptionCatchSample());
            Sampler.Samples.Add(new LoopBreakSample());
            Sampler.Samples.Add(new LoopStopSample());
            Sampler.Samples.Add(new CancellationSample());
            Sampler.Samples.Add(new AsyncAwaitSample());
			ResetSampler();
		}

		private void ResetSampler()
		{
			this.Samples.Clear();
			foreach (var sample in Sampler.Samples)
			{
				this.Samples.Add(CreateSampleViewModel(sample));
			}
		}

		private SampleViewModel CreateSampleViewModel(Sample sample)
		{
			SampleViewModel vm = SamplerViewModelFactory.GetViewModelForSample(sample);
			return vm;
		}

		public void ChangeImagePath()
		{
			OpenFileDialog ofd = new OpenFileDialog();
			ofd.CheckFileExists = true;
			ofd.CheckPathExists = true;
			ofd.Multiselect = false;
			if (ofd.ShowDialog() == true)
			{
				Sampler.ImagePath = ofd.FileName;
			}
		}

		public async void Submit()
		{
			CurrentState = "Running";
			await Sampler.RunSamples();
			CurrentState = "Completed";
		}

		public void Reset()
		{
			Sampler.ResetDestinationImage();
			Sampler.Results = "";
		}

		private class SamplerViewModelFactory
		{
			private static Dictionary<Type, Func<Sample, SampleViewModel>> maps = new Dictionary<Type, Func<Sample, SampleViewModel>>()
            {
				{ typeof(LineSample), (q) => new SampleViewModel((Sample)q)},
				{ typeof(LineParallelSample), (q) => new SampleViewModel((Sample)q)},
				{ typeof(GreyScaleSample), (q) => new SampleViewModel((Sample)q)},
				{ typeof(GreyScaleParallelSample), (q) => new SampleViewModel((Sample)q)},
				{ typeof(GreyScaleDoubleParallelSample), (q) => new SampleViewModel((Sample)q)},
                { typeof(MatricesMultiplicationSample), (q) => new SampleViewModel((Sample)q)},
                { typeof(MatricesMultiplicationParallelSample), (q) => new SampleViewModel((Sample)q)},
                { typeof(AggregateExceptionNoCatchSample), (q) => new SampleViewModel((Sample)q)},
                { typeof(AggregateExceptionCatchSample), (q) => new SampleViewModel((Sample)q)},
                { typeof(LoopBreakSample), (q) => new SampleViewModel((Sample)q)},
                { typeof(LoopStopSample), (q) => new SampleViewModel((Sample)q)},
                { typeof(CancellationSample), (q) => new CancellationSampleViewModel((CancellationSample)q)},
                { typeof(AsyncAwaitSample), (q) => new SampleViewModel((Sample)q)}
            };

			public static SampleViewModel GetViewModelForSample(Sample sample)
			{
				Func<Sample, SampleViewModel> viewModelInstanceFactory = null;
				if (maps.TryGetValue(sample.GetType(), out viewModelInstanceFactory))
				{
					return viewModelInstanceFactory(sample);
				}
				else
				{
					throw new ArgumentOutOfRangeException("Could not locate a view model for sample type");
				}
			}
		}
	}
}
